@@ -4,7 +4,7 @@
 
    Written by - Scott Dattalo scott@dattalo.com
    Ported to PIC16 by - Martin Dubuc m.debuc@rogers.com
-    
+
    Note that mlh prepended _pic16_ on the static functions.  Makes
    it easier to set a breakpoint using the debugger.
 
@@ -13,12 +13,12 @@
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2, or (at your option) any
    later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
@@ -51,7 +51,7 @@ static char *_pic16_keywords[] =
   "far",
   "interrupt",
   "near",
-  "pdata",
+  //"pdata",
   "reentrant",
   "sfr",
   "sfr16",
@@ -60,7 +60,7 @@ static char *_pic16_keywords[] =
   "_code",
   "_generic",
   "_near",
-  "_pdata",
+  //"_pdata",
   "_naked",
   "shadowregs",
   "wparam",
@@ -68,7 +68,7 @@ static char *_pic16_keywords[] =
   "prodhp",
   "fsr0lp",
   "fixed16x16",
-  
+
 //  "bit",
 //  "idata",
 //  "sbit",
@@ -81,16 +81,13 @@ static char *_pic16_keywords[] =
 
 pic16_sectioninfo_t pic16_sectioninfo;
 
-int xinst=0;
-
-
 extern char *pic16_processor_base_name(void);
 
 void  pic16_pCodeInitRegisters(void);
 
 void pic16_assignRegisters (ebbIndex *);
 
-static int regParmFlg = 0;	/* determine if we can register a parameter */
+static int regParmFlg = 0;  /* determine if we can register a parameter */
 
 pic16_options_t pic16_options;
 
@@ -101,7 +98,7 @@ extern set *libFilesSet;
 /* Also defined in gen.h, but the #include is commented out */
 /* for an unknowned reason. - EEP */
 void pic16_emitDebuggerSymbol (char *);
- 
+
 extern void pic16_emitConfigRegs(FILE *of);
 extern void pic16_emitIDRegs(FILE *of);
 
@@ -130,7 +127,7 @@ _pic16_regparm (sym_link * l, bool reentrant)
      * can pass only the first parameter in a register */
     if(regParmFlg)return 0;
       regParmFlg++;
-      return 1;	//regParmFlg;
+      return 1; //regParmFlg;
   } else {
     /* otherwise pass all arguments in registers via SEND/RECEIVE */
     regParmFlg++;// = 1;
@@ -139,12 +136,12 @@ _pic16_regparm (sym_link * l, bool reentrant)
 }
 
 
-int initsfpnt=0;		/* set to 1 if source provides a pragma for stack
-				 * so glue() later emits code to initialize stack/frame pointers */
+int initsfpnt=0;        /* set to 1 if source provides a pragma for stack
+                 * so glue() later emits code to initialize stack/frame pointers */
 set *absSymSet;
 
-set *sectNames=NULL;			/* list of section listed in pragma directives */
-set *sectSyms=NULL;			/* list of symbols set in a specific section */
+set *sectNames=NULL;            /* list of section listed in pragma directives */
+set *sectSyms=NULL;         /* list of symbols set in a specific section */
 set *wparamList=NULL;
 
 #if 0
@@ -160,11 +157,10 @@ struct {
   unsigned want_libio: 1;
   unsigned want_libdebug: 1;
 } libflags = { 0, 0, 0, 0, 0 };
-  
+
 
 enum {
-  P_MAXRAM = 1,
-  P_STACK,
+  P_STACK = 1,
   P_CODE,
   P_UDATA,
   P_LIBRARY
@@ -181,31 +177,6 @@ do_pragma(int id, const char *name, const char *cp)
 
   switch (id)
     {
-    /* #pragma maxram [maxram] */
-    case P_MAXRAM:
-      {
-        int max_ram;
-
-        cp = get_pragma_token(cp, &token);
-        if (TOKEN_INT == token.type)
-          max_ram = token.val.int_val;
-        else
-          {
-            err = 1;
-            break;
-          }
-
-        cp = get_pragma_token(cp, &token);
-        if (TOKEN_EOL != token.type)
-          {
-            err = 1;
-            break;
-          }
-
-        pic16_setMaxRAM(max_ram);
-      }
-      break;
-
     /* #pragma stack [stack-position] [stack-len] */
     case  P_STACK:
       {
@@ -280,7 +251,7 @@ do_pragma(int id, const char *name, const char *cp)
         sym = newSymbol("stack_end", 0);
         sprintf(sym->rname, "_%s", sym->name);
         addSet(&publics, sym);
-    
+
         initsfpnt = 1;    // force glue() to initialize stack/frame pointers */
       }
       break;
@@ -424,12 +395,12 @@ do_pragma(int id, const char *name, const char *cp)
             lmodule = get_pragma_string(&token);
 
             /* lmodule can be:
-             * c	link the C library
-             * math	link the math library
-             * io	link the IO library
-             * debug	link the debug libary
+             * c    link the C library
+             * math link the math library
+             * io   link the IO library
+             * debug    link the debug libary
              * anything else, will link as-is */
-     
+
             if(!strcmp(lmodule, "c"))
               libflags.want_libc = 1;
             else if(!strcmp(lmodule, "math"))
@@ -478,7 +449,7 @@ do_pragma(int id, const char *name, const char *cp)
 
         {
           char *s;
-          
+
           for(s = setFirstItem(asmInlineMap); s ; s = setNextItem(asmInlineMap)) {
             debugf("inline asm: `%s'\n", s);
           }
@@ -502,7 +473,6 @@ do_pragma(int id, const char *name, const char *cp)
 }
 
 static struct pragma_s pragma_tbl[] = {
-  { "maxram",  P_MAXRAM,  0, do_pragma },
   { "stack",   P_STACK,   0, do_pragma },
   { "code",    P_CODE,    0, do_pragma },
   { "udata",   P_UDATA,   0, do_pragma },
@@ -517,89 +487,89 @@ _process_pragma(const char *s)
   return process_pragma_tbl(pragma_tbl, s);
 }
 
-#define REP_UDATA	"--preplace-udata-with="
+#define REP_UDATA         "--preplace-udata-with="
 
-#define STACK_MODEL	"--pstack-model="
-#define OPT_BANKSEL	"--obanksel="
+#define STACK_MODEL       "--pstack-model="
+#define OPT_BANKSEL       "--obanksel="
 
-#define ALT_ASM		"--asm="
-#define ALT_LINK	"--link="
+#define ALT_ASM           "--asm="
+#define ALT_LINK          "--link="
 
-#define IVT_LOC		"--ivt-loc="
-#define NO_DEFLIBS	"--nodefaultlibs"
-#define MPLAB_COMPAT	"--mplab-comp"
+#define IVT_LOC           "--ivt-loc="
+#define NO_DEFLIBS        "--nodefaultlibs"
+#define MPLAB_COMPAT      "--mplab-comp"
 
-#define USE_CRT		"--use-crt="
+#define USE_CRT           "--use-crt="
 
-#define	OFMSG_LRSUPPORT	"--flr-support"
+#define OFMSG_LRSUPPORT   "--flr-support"
 
-#define OPTIMIZE_GOTO   "--optimize-goto"
-#define	OPTIMIZE_CMP	"--optimize-cmp"
-#define	OPTIMIZE_DF	"--optimize-df"
+#define NO_OPTIMIZE_GOTO  "--no-optimize-goto"
+#define OPTIMIZE_CMP      "--optimize-cmp"
+#define OPTIMIZE_DF       "--optimize-df"
 
-char *alt_asm=NULL;
-char *alt_link=NULL;
+char *alt_asm = NULL;
+char *alt_link = NULL;
 
-int pic16_mplab_comp=0;
+int pic16_mplab_comp = 0;
 extern int pic16_debug_verbose;
 extern int pic16_ralloc_debug;
 extern int pic16_pcode_verbose;
 
-int pic16_enable_peeps=0;
+int pic16_enable_peeps = 0;
 
 OPTION pic16_optionsTable[]= {
-	/* code generation options */
-	{ 0,	STACK_MODEL,		NULL,				"use stack model 'small' (default) or 'large'"},
+    /* code generation options */
+    { 0, STACK_MODEL,        NULL, "use stack model 'small' (default) or 'large'"},
 #if XINST
-	{ 'y',  "--extended",   &xinst, "enable Extended Instruction Set/Literal Offset Addressing mode"},
+    { 'y', "--extended",     &pic16_options.xinst, "enable Extended Instruction Set/Literal Offset Addressing mode"},
 #endif
-	{ 0,	"--pno-banksel",	&pic16_options.no_banksel,	"do not generate BANKSEL assembler directives"},
+    { 0, "--pno-banksel",    &pic16_options.no_banksel, "do not generate BANKSEL assembler directives"},
 
-	/* optimization options */
-	{ 0,	OPT_BANKSEL,		NULL,				"set banksel optimization level (default=0 no)"},
-	{ 0,	"--denable-peeps",	&pic16_enable_peeps,	"explicit enable of peepholes"},
-	{ 0,    OPTIMIZE_GOTO,  NULL,			"try to use (conditional) BRA instead of GOTO"},
-	{ 0,	OPTIMIZE_CMP,	NULL,			"try to optimize some compares"},
-	{ 0,	OPTIMIZE_DF,	NULL,			"thoroughly analyze data flow (memory and time intensive!)"},
+    /* optimization options */
+    { 0, OPT_BANKSEL,       &pic16_options.opt_banksel, "set banksel optimization level (default=0 no)", CLAT_INTEGER },
+    { 0, "--denable-peeps", &pic16_enable_peeps, "explicit enable of peepholes"},
+    { 0, NO_OPTIMIZE_GOTO,  NULL, "do NOT use (conditional) BRA instead of GOTO"},
+    { 0, OPTIMIZE_CMP,      NULL, "try to optimize some compares"},
+    { 0, OPTIMIZE_DF,       NULL, "thoroughly analyze data flow (memory and time intensive!)"},
 
-	/* assembling options */
-	{ 0,	ALT_ASM,	NULL,	"Use alternative assembler"},
-	{ 0,	MPLAB_COMPAT,		&pic16_mplab_comp,	"enable compatibility mode for MPLAB utilities (MPASM/MPLINK)"},
+    /* assembling options */
+    { 0, ALT_ASM,           &alt_asm, "Use alternative assembler", CLAT_STRING},
+    { 0, MPLAB_COMPAT,      &pic16_mplab_comp, "enable compatibility mode for MPLAB utilities (MPASM/MPLINK)"},
 
-	/* linking options */
-	{ 0,	ALT_LINK,	NULL,	"Use alternative linker"},
-	{ 0,	REP_UDATA,	NULL,	"Place udata variables at another section: udata_acs, udata_ovr, udata_shr"},
-	{ 0,	IVT_LOC,	NULL,	"Set address of interrupt vector table."},
-	{ 0,	NO_DEFLIBS,		&pic16_options.nodefaultlibs,	"do not link default libraries when linking"},
-	{ 0,	USE_CRT,	NULL,	"use <crt-o> run-time initialization module"},
-	{ 0,	"--no-crt",	&pic16_options.no_crt,	"do not link any default run-time initialization module"},
+    /* linking options */
+    { 0, ALT_LINK,          &alt_link, "Use alternative linker", CLAT_STRING },
+    { 0, REP_UDATA,         &pic16_sectioninfo.at_udata, "Place udata variables at another section: udata_acs, udata_ovr, udata_shr", CLAT_STRING },
+    { 0, IVT_LOC,           NULL, "Set address of interrupt vector table."},
+    { 0, NO_DEFLIBS,        &pic16_options.nodefaultlibs,   "do not link default libraries when linking"},
+    { 0, USE_CRT,           NULL, "use <crt-o> run-time initialization module"},
+    { 0, "--no-crt",        &pic16_options.no_crt, "do not link any default run-time initialization module"},
 
-	/* debugging options */
-	{ 0,	"--debug-xtra",		&pic16_debug_verbose,	"show more debug info in assembly output"},
-	{ 0,	"--debug-ralloc",	&pic16_ralloc_debug,	"dump register allocator debug file *.d"},
-	{ 0,	"--pcode-verbose",	&pic16_pcode_verbose,	"dump pcode related info"},
-	{ 0,	"--calltree",		&pic16_options.dumpcalltree,	"dump call tree in .calltree file"},
-	{ 0,	"--gstack",	&pic16_options.gstack,	"trace stack pointer push/pop to overflow"},
-	{ 0,	NULL,		NULL,	NULL}
+    /* debugging options */
+    { 0, "--debug-xtra",    &pic16_debug_verbose, "show more debug info in assembly output"},
+    { 0, "--debug-ralloc",  &pic16_ralloc_debug, "dump register allocator debug file *.d"},
+    { 0, "--pcode-verbose", &pic16_pcode_verbose, "dump pcode related info"},
+    { 0, "--calltree",      &pic16_options.dumpcalltree, "dump call tree in .calltree file"},
+    { 0, "--gstack",        &pic16_options.gstack, "trace stack pointer push/pop to overflow"},
+    { 0, NULL,              NULL, NULL}
 };
 
 
-#define ISOPT(str)	!strncmp(argv[ *i ], str, strlen(str) )
+#define ISOPT(str)  !strncmp(argv[ *i ], str, strlen(str) )
 
 static bool
 _pic16_parseOptions (int *pargc, char **argv, int *i)
 {
   int j=0;
   char *stkmodel;
-  
+
   /* TODO: allow port-specific command line options to specify
    * segment names here.
    */
-  
+
     /* check for arguments that have associated an integer variable */
     while(pic16_optionsTable[j].pparameter) {
       if(ISOPT( pic16_optionsTable[j].longOpt )) {
-        (*pic16_optionsTable[j].pparameter)++;
+        (*(int *)pic16_optionsTable[j].pparameter)++;
         return TRUE;
       }
       j++;
@@ -616,32 +586,12 @@ _pic16_parseOptions (int *pargc, char **argv, int *i)
       return TRUE;
     }
 
-    if(ISOPT(OPT_BANKSEL)) {
-      pic16_options.opt_banksel = getIntArg(OPT_BANKSEL, argv, i, *pargc);
-      return TRUE;
-    }
-
-    if(ISOPT(REP_UDATA)) {
-      pic16_sectioninfo.at_udata = Safe_strdup(getStringArg(REP_UDATA, argv, i, *pargc));
-      return TRUE;
-    }
-	
-    if(ISOPT(ALT_ASM)) {
-      alt_asm = Safe_strdup(getStringArg(ALT_ASM, argv, i, *pargc));
-      return TRUE;
-    }
-	
-    if(ISOPT(ALT_LINK)) {
-      alt_link = Safe_strdup(getStringArg(ALT_LINK, argv, i, *pargc));
-      return TRUE;
-    }
-
     if(ISOPT(IVT_LOC)) {
       pic16_options.ivt_loc = getIntArg(IVT_LOC, argv, i, *pargc);
       fprintf(stderr, "%s:%d setting interrupt vector addresses 0x%x\n", __FILE__, __LINE__, pic16_options.ivt_loc);
       return TRUE;
     }
-	
+
     if(ISOPT(USE_CRT)) {
       pic16_options.no_crt = 0;
       pic16_options.crt_name = Safe_strdup( getStringArg(USE_CRT, argv, i, *pargc) );
@@ -656,8 +606,8 @@ _pic16_parseOptions (int *pargc, char **argv, int *i)
     }
 #endif
 
-    if (ISOPT(OPTIMIZE_GOTO)) {
-      pic16_options.opt_flags |= OF_OPTIMIZE_GOTO;
+    if (ISOPT(NO_OPTIMIZE_GOTO)) {
+      pic16_options.opt_flags |= OF_NO_OPTIMIZE_GOTO;
       return TRUE;
     }
 
@@ -670,72 +620,41 @@ _pic16_parseOptions (int *pargc, char **argv, int *i)
       pic16_options.opt_flags |= OF_OPTIMIZE_DF;
       return TRUE;
     }
-    
+
 
   return FALSE;
 }
 
-extern set *userIncDirsSet;
+extern void pic16_init_pic(const char *name);
 
 static void _pic16_initPaths(void)
 {
-  set *pic16incDirsSet=NULL;
-  set *pic16libDirsSet=NULL;
-  char devlib[512];
+    set *pic16libDirsSet=NULL;
 
-    setMainValue("mcu", pic16->name[2] );
-    addSet(&preArgvSet, Safe_strdup("-D{mcu}"));
+    if (!options.nostdlib) {
+        struct dbuf_s pic16libDir;
 
-    setMainValue("mcu1", pic16->name[1] );
-    addSet(&preArgvSet, Safe_strdup("-D__{mcu1}"));
-
-    if(!options.nostdinc) {
-      struct dbuf_s pic16incDir;
-
-      dbuf_init(&pic16incDir, 128);
-      dbuf_makePath(&pic16incDir, INCLUDE_DIR_SUFFIX, "pic16");
-
-      /* setup pic16 include directory */
-      pic16incDirsSet = appendStrSet(dataDirsSet, NULL, dbuf_c_str(&pic16incDir));
-      dbuf_destroy(&pic16incDir);
-      includeDirsSet = pic16incDirsSet;
-//      mergeSets(&includeDirsSet, pic16incDirsSet);
-    }
-    /* pic16 port should not search to the SDCC standard include directories,
-     * so add here the deleted include dirs that user has issued in command line */
-    mergeSets(&pic16incDirsSet, userIncDirsSet);
-
-    if(!options.nostdlib) {
-      struct dbuf_s pic16libDir;
-
-      dbuf_init(&pic16libDir, 128);
-      dbuf_makePath(&pic16libDir, LIB_DIR_SUFFIX, "pic16");
-      /* setup pic16 library directory */
-      pic16libDirsSet = appendStrSet(dataDirsSet, NULL, dbuf_c_str(&pic16libDir));
-      dbuf_destroy(&pic16libDir);
-      libDirsSet = pic16libDirsSet;
-//      mergeSets(&libDirsSet, pic16libDirsSet);
+        dbuf_init(&pic16libDir, 128);
+        dbuf_makePath(&pic16libDir, LIB_DIR_SUFFIX, "pic16");
+        pic16libDirsSet = appendStrSet(dataDirsSet, NULL, dbuf_c_str(&pic16libDir));
+        dbuf_destroy(&pic16libDir);
+        mergeSets(&pic16libDirsSet, libDirsSet);
+        libDirsSet = pic16libDirsSet;
     }
 
-    if(!pic16_options.nodefaultlibs) {
-      /* now add the library for the device */
-      sprintf(devlib, "%s.lib", pic16->name[2]);
-      addSet(&libFilesSet, Safe_strdup(devlib));
-
-      /* add the internal SDCC library */
-      addSet(&libFilesSet, Safe_strdup( "libsdcc.lib" ));
-    }
+    /* now that we have the paths set up... */
+    pic16_init_pic(port->processor);
 }
 
 extern set *linkOptionsSet;
 char *msprintf(hTab *pvals, const char *pformat, ...);
 int my_system(const char *cmd);
 
-/* forward declarations */   
+/* forward declarations */
 extern const char *pic16_linkCmd[];
 extern const char *pic16_asmCmd[];
 extern set *asmOptionsSet;
-  
+
 /* custom function to link objects */
 static void _pic16_linkEdit(void)
 {
@@ -745,162 +664,155 @@ static void _pic16_linkEdit(void)
   char temp[1024];
   set *tSet=NULL;
   int ret;
-  
-  	/*
-  	 * link command format:
-  	 * {linker} {incdirs} {lflags} -o {outfile} {spec_ofiles} {ofiles} {libs}
-  	 *
-  	 */
-  	sprintf(lfrm, "{linker} {incdirs} {lflags} -w -r -o {outfile} {user_ofile} {ofiles} {spec_ofiles} {libs}");
 
-  	shash_add(&linkValues, "linker", pic16_linkCmd[0]);
+    /*
+     * link command format:
+     * {linker} {incdirs} {lflags} -o {outfile} {spec_ofiles} {ofiles} {libs}
+     *
+     */
+    sprintf(lfrm, "{linker} {incdirs} {lflags} -w -r -o {outfile} {user_ofile} {ofiles} {spec_ofiles} {libs}");
 
-  	mergeSets(&tSet, libDirsSet);
-  	mergeSets(&tSet, libPathsSet);
-  	
-  	shash_add(&linkValues, "incdirs", joinStrSet( appendStrSet(tSet, "-I\"", "\"")));
-  	shash_add(&linkValues, "lflags", joinStrSet(linkOptionsSet));
-  
-	shash_add(&linkValues, "outfile", fullDstFileName ? fullDstFileName : dstFileName);
+    shash_add(&linkValues, "linker", pic16_linkCmd[0]);
 
-  	if(fullSrcFileName) {
-		sprintf(temp, "%s.o", fullDstFileName ? fullDstFileName : dstFileName);
-//		addSetHead(&relFilesSet, Safe_strdup(temp));
+    mergeSets(&tSet, libPathsSet);
+    mergeSets(&tSet, libDirsSet);
+
+    shash_add(&linkValues, "incdirs", joinStrSet( appendStrSet(tSet, "-I\"", "\"")));
+    shash_add(&linkValues, "lflags", joinStrSet(linkOptionsSet));
+
+    shash_add(&linkValues, "outfile", fullDstFileName ? fullDstFileName : dstFileName);
+
+    if(fullSrcFileName) {
+        sprintf(temp, "%s.o", fullDstFileName ? fullDstFileName : dstFileName);
+//      addSetHead(&relFilesSet, Safe_strdup(temp));
                 shash_add(&linkValues, "user_ofile", temp);
-	}
+    }
 
-	if(!pic16_options.no_crt)
+    if(!pic16_options.no_crt)
           shash_add(&linkValues, "spec_ofiles", pic16_options.crt_name);
 
-  	shash_add(&linkValues, "ofiles", joinStrSet(relFilesSet));
+    shash_add(&linkValues, "ofiles", joinStrSet(relFilesSet));
+
+    if(!libflags.ignore) {
+      if(libflags.want_libc)
+        addSet(&libFilesSet, Safe_strdup("libc18f.lib"));
 
-  	if(!libflags.ignore) {
-  	  if(libflags.want_libc)
-  	    addSet(&libFilesSet, Safe_strdup("libc18f.lib"));
-        
           if(libflags.want_libm)
             addSet(&libFilesSet, Safe_strdup("libm18f.lib"));
-        
+
           if(libflags.want_libio) {
-            sprintf(temp, "libio%s.lib", pic16->name[1]);	/* build libio18f452.lib name */
+            sprintf(temp, "libio%s.lib", pic16->name[1]);   /* build libio18f452.lib name */
             addSet(&libFilesSet, Safe_strdup(temp));
           }
-        
+
           if(libflags.want_libdebug)
             addSet(&libFilesSet, Safe_strdup("libdebug.lib"));
         }
 
-  	shash_add(&linkValues, "libs", joinStrSet(libFilesSet));
-  	
-  	lcmd = msprintf(linkValues, lfrm);
-  	 
-  	ret = my_system( lcmd );
-  	 
-  	Safe_free( lcmd );
-  	 
-  	if(ret)
-  		exit(1);
+    shash_add(&linkValues, "libs", joinStrSet(libFilesSet));
+
+    lcmd = msprintf(linkValues, lfrm);
+
+    ret = my_system( lcmd );
+
+    Safe_free( lcmd );
+
+    if(ret)
+        exit(1);
 }
 
 
 static void
 _pic16_finaliseOptions (void)
 {
-    port->mem.default_local_map = data;
-    port->mem.default_globl_map = data;
+  port->mem.default_local_map = data;
+  port->mem.default_globl_map = data;
 
-    /* peepholes are disabled for the time being */
-    options.nopeep = 1;
+  /* peepholes are disabled for the time being */
+  options.nopeep = 1;
 
-    /* explicit enable peepholes for testing */
-    if(pic16_enable_peeps)
-      options.nopeep = 0;
+  /* explicit enable peepholes for testing */
+  if (pic16_enable_peeps)
+    options.nopeep = 0;
 
-    options.all_callee_saves = 1;		// always callee saves
+  options.all_callee_saves = 1;       // always callee saves
 
 #if 0
-    options.float_rent = 1;
-    options.intlong_rent = 1;
+  options.float_rent = 1;
+  options.intlong_rent = 1;
 #endif
-	
 
-    if(alt_asm && strlen(alt_asm))
+  setMainValue("mcu", pic16->name[2] );
+  addSet(&preArgvSet, Safe_strdup("-D{mcu}"));
+
+  setMainValue("mcu1", pic16->name[1] );
+  addSet(&preArgvSet, Safe_strdup("-D__{mcu1}"));
+
+  if (!pic16_options.nodefaultlibs)
+    {
+      char devlib[512];
+
+      /* now add the library for the device */
+      sprintf(devlib, "libdev%s.lib", pic16->name[1]);   /* e.g., libdev18f452.lib */
+      addSet(&libFilesSet, Safe_strdup(devlib));
+
+      /* add the internal SDCC library */
+      addSet(&libFilesSet, Safe_strdup( "libsdcc.lib" ));
+    }
+
+  if (alt_asm && alt_asm[0] != '\0')
+    {
       pic16_asmCmd[0] = alt_asm;
-	
-    if(alt_link && strlen(alt_link))
+    }
+
+  if (alt_link && alt_link[0] != '\0')
+    {
       pic16_linkCmd[0] = alt_link;
-        
-    if(!pic16_options.no_crt) {
+    }
+
+  if (!pic16_options.no_crt)
+    {
       pic16_options.omit_ivt = 1;
       pic16_options.leave_reset = 0;
     }
-    
-    if(options.model == MODEL_SMALL)
+
+  if (options.model == MODEL_SMALL)
+    {
       addSet(&asmOptionsSet, Safe_strdup("-DSDCC_MODEL_SMALL"));
-    else
-    if(options.model == MODEL_LARGE)
-      addSet(&asmOptionsSet, Safe_strdup("-DSDCC_MODEL_LARGE"));
-    
+    }
+  else if (options.model == MODEL_LARGE)
     {
       char buf[128];
 
-        sprintf(buf, "-D%s -D__%s", pic16->name[2], pic16->name[1]);
-        *(strrchr(buf, 'f')) = 'F';
-        addSet(&asmOptionsSet, Safe_strdup( buf ));
+      addSet(&asmOptionsSet, Safe_strdup("-DSDCC_MODEL_LARGE"));
+
+      sprintf(buf, "-D%s -D__%s", pic16->name[2], pic16->name[1]);
+      *(strrchr(buf, 'f')) = 'F';
+      addSet(&asmOptionsSet, Safe_strdup(buf));
     }
-    
-    if(STACK_MODEL_LARGE) {
+
+  if (STACK_MODEL_LARGE)
+    {
       addSet(&preArgvSet, Safe_strdup("-DSTACK_MODEL_LARGE"));
       addSet(&asmOptionsSet, Safe_strdup("-DSTACK_MODEL_LARGE"));
-    } else {
+    }
+  else
+    {
       addSet(&preArgvSet, Safe_strdup("-DSTACK_MODEL_SMALL"));
       addSet(&asmOptionsSet, Safe_strdup("-DSTACK_MODEL_SMALL"));
     }
 }
 
 
-#if 0
-  if (options.model == MODEL_LARGE)
-    {
-      port->mem.default_local_map = xdata;
-      port->mem.default_globl_map = xdata;
-    }
-  else
-    {
-      port->mem.default_local_map = data;
-      port->mem.default_globl_map = data;
-    }
-
-  if (options.stack10bit)
-    {
-      if (options.model != MODEL_FLAT24)
-	{
-	  fprintf (stderr,
-		   "*** warning: 10 bit stack mode is only supported in flat24 model.\n");
-	  fprintf (stderr, "\t10 bit stack mode disabled.\n");
-	  options.stack10bit = 0;
-	}
-      else
-	{
-	  /* Fixup the memory map for the stack; it is now in
-	   * far space and requires a FPOINTER to access it.
-	   */
-	  istack->fmap = 1;
-	  istack->ptrType = FPOINTER;
-	}
-    }
-#endif
-
-
 static void
 _pic16_setDefaultOptions (void)
 {
-  options.stackAuto = 0;		/* implicit declaration */
-  /* port is not capable yet to allocate separate registers 
+  options.stackAuto = 0;        /* implicit declaration */
+  /* port is not capable yet to allocate separate registers
    * dedicated for passing certain parameters */
-  
+
   /* initialize to defaults section locations, names and addresses */
-  pic16_sectioninfo.at_udata	= "udata";
+  pic16_sectioninfo.at_udata    = "udata";
 
   /* set pic16 port options to defaults */
   pic16_options.no_banksel = 0;
@@ -908,13 +820,13 @@ _pic16_setDefaultOptions (void)
   pic16_options.omit_configw = 0;
   pic16_options.omit_ivt = 0;
   pic16_options.leave_reset = 0;
-  pic16_options.stack_model = 0;			/* 0 for 'small', 1 for 'large' */
+  pic16_options.stack_model = 0;            /* 0 for 'small', 1 for 'large' */
   pic16_options.ivt_loc = 0x000000;
   pic16_options.nodefaultlibs = 0;
   pic16_options.dumpcalltree = 0;
-  pic16_options.crt_name = "crt0i.o";		/* the default crt to link */
-  pic16_options.no_crt = 0;			/* use crt by default */
-  pic16_options.ip_stack = 1;		/* set to 1 to enable ipop/ipush for stack */
+  pic16_options.crt_name = "crt0i.o";       /* the default crt to link */
+  pic16_options.no_crt = 0;         /* use crt by default */
+  pic16_options.ip_stack = 1;       /* set to 1 to enable ipop/ipush for stack */
   pic16_options.gstack = 0;
   pic16_options.debgen = 0;
 }
@@ -931,7 +843,7 @@ _pic16_getRegName (struct regs *reg)
 #if 1
 static  char *_pic16_mangleFunctionName(char *sz)
 {
-//	fprintf(stderr, "mangled function name: %s\n", sz);
+//  fprintf(stderr, "mangled function name: %s\n", sz);
 
   return sz;
 }
@@ -943,23 +855,23 @@ _pic16_genAssemblerPreamble (FILE * of)
 {
   char *name = pic16_processor_base_name();
 
-	if(!name) {
-		name = "p18f452";
-		fprintf(stderr,"WARNING: No Pic has been selected, defaulting to %s\n",name);
-	}
+    if(!name) {
+        name = "p18f452";
+        fprintf(stderr,"WARNING: No Pic has been selected, defaulting to %s\n",name);
+    }
 
-	fprintf (of, "\tlist\tp=%s\n",&name[1]);
-	if (pic16_mplab_comp) {
-	  // provide ACCESS macro used during SFR accesses
-	  fprintf (of, "\tinclude <p%s.inc>\n", &name[1]);
-	}
+    fprintf (of, "\tlist\tp=%s\n",&name[1]);
+    if (pic16_mplab_comp) {
+      // provide ACCESS macro used during SFR accesses
+      fprintf (of, "\tinclude <p%s.inc>\n", &name[1]);
+    }
+
+    if(!pic16_options.omit_configw) {
+        pic16_emitConfigRegs(of);
+        fprintf(of, "\n");
+        pic16_emitIDRegs(of);
+    }
 
-	if(!pic16_options.omit_configw) {
-		pic16_emitConfigRegs(of);
-		fprintf(of, "\n");
-		pic16_emitIDRegs(of);
-	}
-	
   fprintf (of, "\tradix dec\n");
 }
 
@@ -968,34 +880,34 @@ static int
 _pic16_genIVT (struct dbuf_s * oBuf, symbol ** interrupts, int maxInterrupts)
 {
 #if 1
-	/* PIC18F family has only two interrupts, the high and the low
-	 * priority interrupts, which reside at 0x0008 and 0x0018 respectively - VR */
+    /* PIC18F family has only two interrupts, the high and the low
+     * priority interrupts, which reside at 0x0008 and 0x0018 respectively - VR */
 
-	if((!pic16_options.omit_ivt) || (pic16_options.omit_ivt && pic16_options.leave_reset)) {
-		dbuf_printf(oBuf, "; RESET vector\n");
-		dbuf_printf(oBuf, "\tgoto\t__sdcc_gsinit_startup\n");
-	}
-	
-	if(!pic16_options.omit_ivt) {
-		dbuf_printf(oBuf, "\tres 4\n");
+    if((!pic16_options.omit_ivt) || (pic16_options.omit_ivt && pic16_options.leave_reset)) {
+        dbuf_printf(oBuf, "; RESET vector\n");
+        dbuf_printf(oBuf, "\tgoto\t__sdcc_gsinit_startup\n");
+    }
+
+    if(!pic16_options.omit_ivt) {
+        dbuf_printf(oBuf, "\tres 4\n");
 
 
-		dbuf_printf(oBuf, "; High priority interrupt vector 0x0008\n");
-		if(interrupts[1]) {
-			dbuf_printf(oBuf, "\tgoto\t%s\n", interrupts[1]->rname);
-			dbuf_printf(oBuf, "\tres\t12\n"); 
-		} else {
-			dbuf_printf(oBuf, "\tretfie\n");
-			dbuf_printf(oBuf, "\tres\t14\n");
-		}
+        dbuf_printf(oBuf, "; High priority interrupt vector 0x0008\n");
+        if(interrupts[1]) {
+            dbuf_printf(oBuf, "\tgoto\t%s\n", interrupts[1]->rname);
+            dbuf_printf(oBuf, "\tres\t12\n");
+        } else {
+            dbuf_printf(oBuf, "\tretfie\n");
+            dbuf_printf(oBuf, "\tres\t14\n");
+        }
 
-		dbuf_printf(oBuf, "; Low priority interrupt vector 0x0018\n");
-		if(interrupts[2]) {
-			dbuf_printf(oBuf, "\tgoto\t%s\n", interrupts[2]->rname);
-		} else {
-			dbuf_printf(oBuf, "\tretfie\n");
-		}
-	}
+        dbuf_printf(oBuf, "; Low priority interrupt vector 0x0018\n");
+        if(interrupts[2]) {
+            dbuf_printf(oBuf, "\tgoto\t%s\n", interrupts[2]->rname);
+        } else {
+            dbuf_printf(oBuf, "\tretfie\n");
+        }
+    }
 #endif
   return TRUE;
 }
@@ -1050,26 +962,26 @@ static bool _hasNativeMulFor (iCode *ic, sym_link *left, sym_link *right)
     /* also allow literals [-128..256) for left/right operands */
     if (IS_VALOP(IC_LEFT(ic)))
     {
-      long l = (long)floatFromVal( OP_VALUE( IC_LEFT(ic) ) );
+      long l = (long) ulFromVal ( OP_VALUE( IC_LEFT(ic) ) );
       sizeL = 4;
       //printf( "%s: val(left) = %ld\n", __FUNCTION__, l );
       if (l >= -128 && l < 256)
       {
-	sizeL = 1;
+    sizeL = 1;
       } else {
-	//printf( "%s: left value %ld outside [-128..256)\n", __FUNCTION__, l );
+    //printf( "%s: left value %ld outside [-128..256)\n", __FUNCTION__, l );
       }
     }
     if (IS_VALOP( IC_RIGHT(ic) ))
     {
-      long l = (long)floatFromVal( OP_VALUE( IC_RIGHT(ic) ) );
+      long l = (long) ulFromVal ( OP_VALUE( IC_RIGHT(ic) ) );
       sizeR = 4;
       //printf( "%s: val(right) = %ld\n", __FUNCTION__, l );
       if (l >= -128 && l < 256)
       {
-	sizeR = 1;
+    sizeR = 1;
       } else {
-	//printf( "%s: right value %ld outside [-128..256)\n", __FUNCTION__, l );
+    //printf( "%s: right value %ld outside [-128..256)\n", __FUNCTION__, l );
       }
     }
 
@@ -1087,26 +999,26 @@ static bool _hasNativeMulFor (iCode *ic, sym_link *left, sym_link *right)
     /* What about literals? */
     if (IS_VALOP( IC_LEFT(ic) ))
     {
-      long l = (long)floatFromVal( OP_VALUE( IC_LEFT(ic) ) );
+      long l = (long) ulFromVal ( OP_VALUE( IC_LEFT(ic) ) );
       sizeL = 4;
       //printf( "%s: val(left) = %ld\n", __FUNCTION__, l );
       if (l >= -128 && l < 256)
       {
-	sizeL = 1;
+    sizeL = 1;
       } else {
-	//printf( "%s: left value %ld outside [-128..256)\n", __FUNCTION__, l );
+    //printf( "%s: left value %ld outside [-128..256)\n", __FUNCTION__, l );
       }
     }
     if (IS_VALOP( IC_RIGHT(ic) ))
     {
-      long l = (long)floatFromVal( OP_VALUE( IC_RIGHT(ic) ) );
+      long l = (long) ulFromVal ( OP_VALUE( IC_RIGHT(ic) ) );
       sizeR = 4;
       //printf( "%s: val(right) = %ld\n", __FUNCTION__, l );
       if (l >= -128 && l < 256)
       {
-	sizeR = 1;
+    sizeR = 1;
       } else {
-	//printf( "%s: right value %ld outside [-128..256)\n", __FUNCTION__, l );
+    //printf( "%s: right value %ld outside [-128..256)\n", __FUNCTION__, l );
       }
     }
     if (sizeL == 1 && sizeR == 1) { return TRUE; }
@@ -1124,16 +1036,16 @@ static bool cseCostEstimation (iCode *ic, iCode *pdic)
 //    sym_link *result_type = operandType(result);
 
 
-	/* VR -- this is an adhoc. Put here after conversation
-	 * with Erik Epetrich */
+    /* VR -- this is an adhoc. Put here after conversation
+     * with Erik Epetrich */
 
-	if(ic->op == '<'
-		|| ic->op == '>'
-		|| ic->op == EQ_OP) {
+    if(ic->op == '<'
+        || ic->op == '>'
+        || ic->op == EQ_OP) {
 
-		fprintf(stderr, "%d %s\n", __LINE__, __FUNCTION__);
-	  return 0;
-	}
+        fprintf(stderr, "%d %s\n", __LINE__, __FUNCTION__);
+      return 0;
+    }
 
 #if 0
     /* if it is a pointer then return ok for now */
@@ -1177,11 +1089,11 @@ oclsExpense (struct memmap *oclass)
   /* free to remove this test -- EEP */
   if (IN_FARSPACE(oclass))
     return 1;
-    
+
   return 0;
 }
 
-/** $1 is the input object file (PIC16 specific)	// >>always the basename<<.
+/** $1 is the input object file (PIC16 specific)    // >>always the basename<<.
     $2 is always the output file.
     $3 -L path and -l libraries
     $l is the list of extra options that should be there somewhere...
@@ -1211,91 +1123,91 @@ PORT pic16_port =
 {
   TARGET_ID_PIC16,
   "pic16",
-  "MCU PIC16",			/* Target name */
-  "p18f452",                    /* Processor */
+  "MCU PIC16",      /* Target name */
+  "p18f452",        /* Processor */
   {
     pic16glue,
-    TRUE,			/* Emit glue around main */
+    TRUE,           /* Emit glue around main */
     MODEL_SMALL | MODEL_LARGE | MODEL_FLAT24,
     MODEL_SMALL
   },
   {
-    pic16_asmCmd,		/* assembler command and arguments */
-    NULL,			/* alternate macro based form */
-    "-g",			/* arguments for debug mode */
-    NULL,			/* arguments for normal mode */
-    0,				/* print externs as global */
-    ".asm",			/* assembler file extension */
-    NULL			/* no do_assemble function */
+    pic16_asmCmd,   /* assembler command and arguments */
+    NULL,           /* alternate macro based form */
+    "-g",           /* arguments for debug mode */
+    NULL,           /* arguments for normal mode */
+    0,              /* print externs as global */
+    ".asm",         /* assembler file extension */
+    NULL            /* no do_assemble function */
   },
   {
-    NULL,			//    pic16_linkCmd,		/* linker command and arguments */
-    NULL,			/* alternate macro based form */
-    _pic16_linkEdit,		//NULL,			/* no do_link function */
-    ".o",			/* extension for object files */
-    0				/* no need for linker file */
+    NULL,           //    pic16_linkCmd,        /* linker command and arguments */
+    NULL,           /* alternate macro based form */
+    _pic16_linkEdit,        //NULL,         /* no do_link function */
+    ".o",           /* extension for object files */
+    0               /* no need for linker file */
   },
   {
     _defaultRules
   },
   {
-	/* Sizes */
-    1,		/* char */
-    2,		/* short */
-    2,		/* int */
-    4,		/* long */
-    2,		/* ptr */
-    3,		/* fptr, far pointers (see Microchip) */
-    3,		/* gptr */
-    1,		/* bit */
-    4,		/* float */
-    4		/* max */
+    /* Sizes */
+    1,      /* char */
+    2,      /* short */
+    2,      /* int */
+    4,      /* long */
+    2,      /* ptr */
+    3,      /* fptr, far pointers (see Microchip) */
+    3,      /* gptr */
+    1,      /* bit */
+    4,      /* float */
+    4       /* max */
   },
 
     /* generic pointer tags */
   {
-    0x00,	/* far */
-    0x80,	/* near */
-    0x00,	/* xstack */
-    0x00	/* code */
+    0x00,   /* far */
+    0x80,   /* near */
+    0x00,   /* xstack */
+    0x00    /* code */
   },
-  
+
   {
-    "XSEG    (XDATA)",		// xstack
-    "STACK   (DATA)",		// istack
-    "CSEG    (CODE)",		// code
-    "DSEG    (DATA)",		// data
-    "ISEG    (DATA)",		// idata
-    "PSEG    (DATA)",		// pdata
-    "XSEG    (XDATA)",		// xdata
-    "BSEG    (BIT)",		// bit
-    "RSEG    (DATA)",		// reg
-    "GSINIT  (CODE)",		// static
-    "OSEG    (OVR,DATA)",	// overlay
-    "GSFINAL (CODE)",		// post static
-    "HOME    (CODE)",	// home
-    NULL,			// xidata
-    NULL,			// xinit
-    "CONST   (CODE)",		// const_name - const data (code or not)
-    "CABS    (ABS,CODE)",	// cabs_name - const absolute data (code or not)
-    "XABS    (ABS,XDATA)",	// xabs_name - absolute xdata
-    "IABS    (ABS,DATA)",	// iabs_name - absolute data
-    NULL,			// default location for auto vars
-    NULL,			// default location for global vars
-    1				// code is read only 1=yes
+    "XSEG    (XDATA)",      // xstack
+    "STACK   (DATA)",       // istack
+    "CSEG    (CODE)",       // code
+    "DSEG    (DATA)",       // data
+    "ISEG    (DATA)",       // idata
+    "PSEG    (DATA)",       // pdata
+    "XSEG    (XDATA)",      // xdata
+    "BSEG    (BIT)",        // bit
+    "RSEG    (DATA)",       // reg
+    "GSINIT  (CODE)",       // static
+    "OSEG    (OVR,DATA)",   // overlay
+    "GSFINAL (CODE)",       // post static
+    "HOME    (CODE)",       // home
+    NULL,                   // xidata
+    NULL,                   // xinit
+    "CONST   (CODE)",       // const_name - const data (code or not)
+    "CABS    (ABS,CODE)",   // cabs_name - const absolute data (code or not)
+    "XABS    (ABS,XDATA)",  // xabs_name - absolute xdata
+    "IABS    (ABS,DATA)",   // iabs_name - absolute data
+    NULL,                   // default location for auto vars
+    NULL,                   // default location for global vars
+    1                       // code is read only 1=yes
   },
   {
-    NULL,		/* genExtraAreaDeclaration */
-    NULL		/* genExatrAreaLinkOptions */
+    NULL,       /* genExtraAreaDeclaration */
+    NULL        /* genExatrAreaLinkOptions */
   },
   {
-	/* stack related information */
-    -1,			/* -1 stack grows downwards, +1 upwards */
-    1,			/* extra overhead when calling between banks */
-    4,			/* extra overhead when the function is an ISR */
-    1,			/* extra overhead for a function call */
-    1,			/* re-entrant space */
-    0			/* 'banked' call overhead, mild overlap with bank_overhead */
+    /* stack related information */
+    -1,         /* -1 stack grows downwards, +1 upwards */
+    1,          /* extra overhead when calling between banks */
+    4,          /* extra overhead when the function is an ISR */
+    1,          /* extra overhead for a function call */
+    1,          /* re-entrant space */
+    0           /* 'banked' call overhead, mild overlap with bank_overhead */
   },
     /* pic16 has an 8 bit mul */
   {
@@ -1325,30 +1237,30 @@ PORT pic16_port =
   _pic16_getRegName,
   _pic16_keywords,
   _pic16_genAssemblerPreamble,
-  NULL,				/* no genAssemblerEnd */
+  NULL,             /* no genAssemblerEnd */
   _pic16_genIVT,
   NULL, // _pic16_genXINIT
-  NULL, 			/* genInitStartup */
+  NULL,             /* genInitStartup */
   _pic16_reset_regparm,
   _pic16_regparm,
-  _process_pragma,				/* process a pragma */
-  _pic16_mangleFunctionName,				/* mangles function name */
+  _process_pragma,  /* process a pragma */
+  _pic16_mangleFunctionName, /* mangles function name */
   _hasNativeMulFor,
-  hasExtBitOp,			/* hasExtBitOp */
-  oclsExpense,			/* oclsExpense */
-  FALSE,			
-  TRUE,				/* little endian */
-  0,				/* leave lt */
-  0,				/* leave gt */
-  1,				/* transform <= to ! > */
-  1,				/* transform >= to ! < */
-  1,				/* transform != to !(a == b) */
-  0,				/* leave == */
-  FALSE,                        /* No array initializer support. */
-  0,	//cseCostEstimation,            /* !!!no CSE cost estimation yet */
-  NULL, 			/* no builtin functions */
-  GPOINTER,			/* treat unqualified pointers as "generic" pointers */
-  1,				/* reset labelKey to 1 */
-  1,				/* globals & local static allowed */
+  hasExtBitOp,      /* hasExtBitOp */
+  oclsExpense,      /* oclsExpense */
+  FALSE,
+  TRUE,             /* little endian */
+  0,                /* leave lt */
+  0,                /* leave gt */
+  1,                /* transform <= to ! > */
+  1,                /* transform >= to ! < */
+  1,                /* transform != to !(a == b) */
+  0,                /* leave == */
+  FALSE,            /* No array initializer support. */
+  0,    //cseCostEstimation,            /* !!!no CSE cost estimation yet */
+  NULL,             /* no builtin functions */
+  GPOINTER,         /* treat unqualified pointers as "generic" pointers */
+  1,                /* reset labelKey to 1 */
+  1,                /* globals & local static allowed */
   PORT_MAGIC
 };
